import React, {useEffect, useState} from "react";
import {ActivityIndicator, FlatList, Platform, StyleSheet, useColorScheme, View} from "react-native";
import {Colors} from "@/constants/Colors";
import {useFunction} from "@/constants/fetchData";
import {usePatient} from "@/hooks/usePatient";
import {SearchBar} from "@/components/SearchBar";
import {searchAppoints} from "@/constants/searchData";
import {AppointmentCard} from "@/components/AppointmentCard";
import * as Calendars from "expo-calendar";
import {Calendar} from "react-native-calendars";

export default function Appointments() {
    const themeStyle = useColorScheme() === "dark" ? Colors.dark : Colors.light;
    const {patientID} = usePatient();
    const [data, setData] = useState([]);
    useFunction(patientID, setData, "getAppointments");
    const [markedDates, setMarkedDates] = useState({});

    useEffect(() => {
        if (data) {
            let updatedEvents = { ...markedDates };

            data.forEach((item) => {
                let appointmentDate = item.appointmentDate;
                if (appointmentDate) {
                    updatedEvents[appointmentDate] = {
                        marked: true,
                        markedColor: themeStyle.color,
                        dotColor: themeStyle.color,
                    };
                }
            });
            setMarkedDates(updatedEvents);
        }
    }, [data]);

    useEffect(() => {
        handleDeviceCalendar()
            .then(response => {
                if(!response){
                    console.log("Calendar is empty.");
                    return;
                }

                let updatedEvents = {...markedDates };
                response?.forEach(res => {
                    const newEvent = typeof res.startDate === "string" ? res.startDate.split("T") : [0];
                    updatedEvents[newEvent] = {
                        marked: true,
                        dotColor: themeStyle.color,
                        textColor: themeStyle.color,
                    };
                })
                setMarkedDates(updatedEvents);
            })
            .catch(error => console.error(error));
    }, [patientID]);

    async function handleDeviceCalendar() {
        const permission = await Calendars.requestCalendarPermissionsAsync();
        if(!permission.granted){
            alert('Permission to access calendar is required');
            return;
        }
        const calendars = await Calendars.getCalendarsAsync();
        const calendarIDs = calendars.map(calendar => calendar.id);

        if(Platform.OS === 'ios'){
            const reminders = await Calendars.getRemindersAsync(calendarIDs, Calendars.ReminderStatus, new Date(), new Date() + 365);
            console.log("Reminders: ", reminders);
        }
        const endDate = new Date();
        endDate.setDate(endDate.getDate() + 365);
        return await Calendars.getEventsAsync(calendarIDs, new Date(), endDate);
    }

    const [search, setSearch] = useState("");
    const [loading, setLoading] = useState(true);
    const [appointmentDate, setAppointmentDate] = useState();
    const [appointmentTime, setAppointmentTime] = useState();
    const [title, setTitle] = useState();
    const [description, setDescription] = useState();
    const [location, setLocation] = useState();

    useEffect(() => {
        if(!patientID || !search){
            setSearch("");
        }
    }, [patientID, search]);

    return (
        <FlatList
            ListHeaderComponent={
            <>
                <Calendar
                    horizontal={true}
                    enableSwipeMonths={true}
                    minDate={new Date().toISOString()}
                    selected={new Date().toISOString()}
                    markingType={"dot"}
                    markedDates={markedDates}
                    style={style.calendarContainer}
                    theme={{
                        backgroundColor: themeStyle.backgroundColor,
                        calendarBackground: themeStyle.backgroundColor,
                        textSectionTitleColor: themeStyle.color,
                        monthTextColor: themeStyle.color,
                        arrowColor: themeStyle.color,
                        todayBackgroundColor: "lightblue",
                        dayTextColor: themeStyle.color,
                        textDisabledColor: "gray",
                    }}
                />
                <SearchBar
                    icon={true}
                    placeholder={"Search Appointment Title."}
                    value={search}
                    onChangeText={setSearch}
                    onBlur={() => searchAppoints(patientID, search, setLoading, setAppointmentDate, setAppointmentTime, setTitle, setDescription, setLocation)}
                />

                {search ? (
                    <ActivityIndicator
                        size={"small"}
                        animating={loading}
                    />
                ): null}

                {search && appointmentDate && appointmentTime && title && description && location ? (
                    <>
                        <AppointmentCard
                            data1={appointmentDate}
                            data2={appointmentTime}
                            item1={title}
                            item2={description}
                            item3={location}
                        />
                        <View style={themeStyle.divider} />
                    </>
                ) : null}
            </>
            }
            nestedScrollEnabled={true}
            data={data}
            style={themeStyle}
            renderItem={({ item }) => (
                search && appointmentTime && item["appointmentTime"] === appointmentTime ? null : (
                    <AppointmentCard
                        data1={item["appointmentDate"]}
                        data2={item["appointmentTime"]}
                        item1={item["title"]}
                        item2={item["description"]}
                        item3={item["location"]}
                        request={item["appointmentID"]}
                        cancel={item["appointmentID"]}
                    />
                )
            )
        }
            keyExtractor={(item, index) => index.toString()}
        />
    );
}

const style = StyleSheet.create({
    calendarContainer:{
        borderWidth: 1,
        borderTopLeftRadius: 75,
        borderTopRightRadius: 75,
        borderBottomLeftRadius: 10,
        borderBottomRightRadius: 20,
        margin: "2%",
        padding: "2%",
    },
})
