import {
    FlatList,
    SectionList,
    StyleProp,
    StyleSheet,
    Text,
    TextStyle,
    useColorScheme,
    View,
    ViewStyle
} from "react-native";
import React, {useEffect, useState} from "react";
import {logOutDoctor, pushDoctorInsert, pushDoctorUpdate} from "@/constants/pushUser";
import {useDoctor} from "@/hooks/useDoctor";
import {DoctorNav} from "@/components/DoctorNav";
import {Colors} from "@/constants/Colors";
import {ListPatientCard} from "@/components/ListPatientCard";
import {SearchBar} from "@/components/SearchBar";
import {Card} from "@/components/Card";
import {fetchRequest} from "@/constants/fetchData";

async function getConnectedPatientNames(doctorID) {
    const formData = new FormData();
    formData.append("action", "getConnectedPatients");
    formData.append("doctorID", doctorID);
    return await fetchRequest(formData);
}

async function getPatientID(patientName) {
    const formData = new FormData();
    formData.append("action", "getPatientID");
    formData.append("patientName", patientName);
    return await fetchRequest(formData);
}

async function getConnectedPatientData(patientID, doctorID, functionName) {
    const formData = new FormData();
    formData.append("action", functionName);
    formData.append("patientID", patientID);
    formData.append("doctorID", doctorID);
    return await fetchRequest(formData);
}

function getDoctorData(search, patientID, doctorID, setPatientID, setDocResult, setAppointResult, setMedsResult, setNotifsResult, setSymptomsResult){
    getPatientID(search).then((response) => {
        if(!response.success){
            console.log("Failed to fetch patientID:", response.message);
            return;
        }
        const patientNumber = parseInt(response.data.patientID, 10);
        setPatientID(patientNumber);
    });
    getConnectedPatientData(patientID, doctorID, "getPatientDocuments").then((response) => {
        if(!response.success){
            console.log("Failed to fetch data:", response.message);
            return;
        }
        setDocResult(response.data);
    });
    getConnectedPatientData(patientID, doctorID, "getPatientAppointments").then((response) => {
        if(!response.success){
            console.log("Failed to fetch data:", response.message);
            return;
        }
        setAppointResult(response.data);
    });
    getConnectedPatientData(patientID, doctorID, "getPatientMedications").then((response) => {
        if(!response.success){
            console.log("Failed to fetch data:", response.message);
            return;
        }
        setMedsResult(response.data);
    });
    getConnectedPatientData(patientID, doctorID, "getPatientNotifications").then((response) => {
        if(!response.success){
            console.log("Failed to fetch data:", response.message);
            return;
        }
        setNotifsResult(response.data);
    });
    getConnectedPatientData(patientID, doctorID, "getPatientSymptoms").then((response) => {
        if(!response.success){
            console.log("Failed to fetch data:", response.message);
            return;
        }
        setSymptomsResult(response.data);
    });
}

function displaySearch(result, titleStyle, title, image, item1, item2, item3){
    return (
        result ? (
            <FlatList
                ListHeaderComponent={
                    <Text style={titleStyle}>{title}</Text>
                }
                data={result}
                keyExtractor={(item, index) => index.toString()}
                renderItem={({ item }) => (
                    <Card
                        image={image}
                        item1={item[item1]}
                        item2={item[item2]}
                        item3={item[item3]}
                    />
                )}
            />
        ) : null
    )
}

export default function Doctor() {
    const themeStyle = useColorScheme() === "dark" ? Colors.dark : Colors.light;
    const [search, setSearch ] = useState();
    const [docResult, setDocResult] = useState([]);
    const [appointResult, setAppointResult] = useState([]);
    const [medsResult, setMedsResult] = useState([]);
    const [notifsResult, setNotifsResult] = useState([]);
    const [symptomsResult, setSymptomsResult] = useState([]);

    const [patientID, setPatientID] = useState(0);
    const {doctorID, setDoctorID} = useDoctor();
    const [patientNames, setPatientNames] = useState([]);

    useEffect(() => {
        getConnectedPatientNames(doctorID)
            .then(response => {
                setPatientNames(response.data);
            });
    }, [doctorID]);

    const sections = [
        {
            title: 'Documents',
            data: [{functionName: "getDoctorDocuments",
                image: "import-contacts",
                dataPoint1: "documentTitle",
                dataPoint2: "documentDescription",
            }],
        },
        {
            title: 'Appointments',
            data: [{ functionName: "getDoctorAppointments",
                image: "access-time-filled",
                column1: "appointmentDate",
                column2: "appointmentTime",
                dataPoint1: "title",
                dataPoint2: "description",
                dataPoint3: "location",
            }],
        },
        {
            title: 'Medications',
            data: [{ functionName: "getDoctorMedications",
                image: "medication",
                dataPoint1: "medicationName",
                dataPoint2: "strength",
                dataPoint3: "type",
                quantity: "quantityAvailable",
                label: "Reorder",
            }],
        },
        {
            title: 'Notifications',
            data: [{ functionName: "getDoctorNotifications",
                image: "notifications",
                dataPoint1: "notificationTime",
                dataPoint2: "subject",
                dataPoint3: "description",
            }],
        },
        {
            title: 'Symptoms',
            data: [{functionName: "getSymptoms",
                image: "assessment",
                dataPoint1: "symptomName",
                dataPoint2: "symptomDescription",
                quantity: "symptomSeverity",
                label: "Severity",
            }],
        },
    ];

    const titleStyle: StyleProp<TextStyle> = {
        fontSize: 18,
        color: themeStyle.color,
        fontWeight: "bold",
        textDecorationLine: "underline",
        alignSelf: "center",
    };

    const textStyle: StyleProp<TextStyle> = {
        color: themeStyle.color,
        alignSelf: "center",
        fontSize: 16,
    };

    const viewStyle: StyleProp<ViewStyle> = {
        backgroundColor: themeStyle.backgroundColor,
        color: themeStyle.color,
        paddingTop: "10%",
    }

    return (
        <SectionList
            ListHeaderComponent={
            <>
                <SearchBar
                    icon={true}
                    placeholder={"Search Patient Name."}
                    value={search}
                    onChangeText={setSearch}
                    onBlur={()=> getDoctorData(search, patientID, doctorID, setPatientID, setDocResult, setAppointResult, setMedsResult, setNotifsResult, setSymptomsResult)}
                />

                {search ? displaySearch(docResult, titleStyle, "Documents", "import-contacts", "documentTitle", "documentDescription", "documentFile") : null}
                {search ? displaySearch(appointResult, titleStyle, "Appointments", "access-time-filled", "title", "description", "location") : null}
                {search ? displaySearch(medsResult, titleStyle, "Medications", "medication", "medicationName", "strength", "type") : null}
                {search ? displaySearch(notifsResult, titleStyle, "Notifications", "notifications", "notificationTime", "subject", "description") : null}
                {search ? displaySearch(symptomsResult, titleStyle, "Symptoms", "assessment", "symptomName", "symptomDescription", "symptomSeverity") : null}

                <DoctorNav
                    buttonText1={"Go to Insert Data"}
                    buttonFunction1={pushDoctorInsert}
                    buttonStyle1={{backgroundColor: themeStyle.moveButton.backgroundColor}}

                    buttonText2={"Sign Out"}
                    buttonFunction2={() => logOutDoctor(setDoctorID)}
                    buttonStyle2={{flexGrow: 1, backgroundColor: themeStyle.cancelButton.backgroundColor}}

                    buttonText3={"Go to Update Data"}
                    buttonFunction3={pushDoctorUpdate}
                    buttonStyle3={{backgroundColor: themeStyle.moveButton.backgroundColor}}
                />
                {patientNames ? (
                    <>
                        <Text style={titleStyle}>Patient Names</Text>
                        <FlatList
                            data={patientNames}
                            keyExtractor={(item, index) => index.toString()}
                            renderItem={({ item }) => (
                                <Text style={textStyle}>- {item["patientName"]}</Text>
                            )}
                        />
                    </>
                ) : null}
            </>
            }
            stickySectionHeadersEnabled={true}
            style={viewStyle}
            sections={sections}
            renderItem={({ item }) => (
                <ListPatientCard
                    image={item.image}
                    functionName={item.functionName}
                    dataPoint1={item.dataPoint1}
                    dataPoint2={item.dataPoint2}
                    dataPoint3={item.dataPoint3}
                    quantity={item.quantity}
                    label={item.label}
                />
            )}
            renderSectionHeader={({ section }) => (
                <View style={[themeStyle, style.headerContainer]}>
                    <Text style={titleStyle}>
                        {section.title}
                    </Text>
                </View>
            )}
            keyExtractor={(item, index) => index.toString()}
        />
    )
}



const style = StyleSheet.create({
    page: {
        paddingTop: 5,
        paddingLeft: 5,
        paddingRight: 5,
    },
    headerContainer: {
        padding: 10,
    },
});