import React, {useEffect, useState} from "react";
import {ActivityIndicator, Alert, FlatList, Image, Pressable, StyleSheet, useColorScheme, View} from "react-native";
import {SearchBar} from "@/components/SearchBar";
import MaterialIcons from "@expo/vector-icons/MaterialIcons";
import {usePatient} from "@/hooks/usePatient";
import {Colors} from "@/constants/Colors";
import {MediaModal} from "@/components/MediaModal";
import {searchDocuments} from "@/constants/searchData";
import {Card} from "@/components/Card";
import {DocumentCard} from "@/components/DocumentCard";
import {useFunction} from "@/constants/fetchData";
import {insertDocumentFetch} from "@/constants/insertData";

export default function Documents() {
    const themeStyle = useColorScheme() === "dark" ? Colors.dark : Colors.light;
    const { patientID } = usePatient();
    const [data, setData] = useState([]);
    useFunction(patientID, setData, "getDocuments");

    const [search, setSearch] = useState("");
    const [loading, setLoading] = useState(true);
    const [docTitle, setDocTitle] = useState();
    const [docDesc, setDocDesc] = useState();
    const [docType, setDocType] = useState();
    const [followUp, setFollowUp] = useState();
    const [docFile, setDocFile] = useState();

    let [image, setImage] = useState<string | null>(null);
    const [isModalOpen, setIsModalOpen] = useState(false);

    useEffect(() => {
        if(image) {
            confirmUpload(patientID, image);
        }
    }, [image]);

    useEffect(() => {
        setSearch("");
    }, [patientID]);

    return (
        <FlatList
            ListHeaderComponent={
            <>
                <SearchBar
                    icon={true}
                    placeholder={"Search Document Title."}
                    value={search}
                    onChangeText={setSearch}
                    onBlur={() => searchDocuments(patientID, search, setLoading, setDocTitle, setDocDesc, setDocType, setDocFile, setFollowUp)}
                />

                {search ? (
                    <ActivityIndicator
                        size={"small"}
                        animating={loading}
                    />
                ): null}

                {search && docTitle && docDesc && docType && docFile && followUp ? (
                    <>
                        <DocumentCard
                            image={"import-contacts"}
                            item1={docTitle}
                            item2={docDesc}
                            item3={docFile}
                            quantity={followUp}
                            label={"Follow Up"}
                        />
                        <View style={themeStyle.divider} />
                    </>
                ) : null}
            </>
            }
            nestedScrollEnabled={true}
            data={data}
            style={themeStyle}
            renderItem={({ item }) => (
                search && docTitle && item["documentTitle"] === docTitle ? null : (
                    <DocumentCard
                        image={"import-contacts"}
                        item1={item["documentTitle"]}
                        item2={item["documentDescription"]}
                        item3={item["documentFile"]}
                        quantity={item["followUp"]}
                        label={"Follow Up"}
                    />
                )
            )}
            keyExtractor={(item, index) => index.toString()}
            ListFooterComponent={
            <>
                {image ? (
                    <Card item2={
                        <Image
                            source={image ? { uri: image } : require('@/assets/images/placeholder.png')}
                            style={{ width: 200, height: 200, marginTop: 10 }}
                        />
                    }
                    />
                ) : null}

                <Pressable
                    onPress={() => setIsModalOpen(true)}
                    style={[style.floatingButton, {backgroundColor: themeStyle.selectButton.backgroundColor}]}
                >
                    <MaterialIcons name="add" size={100} style={style.floatingButtonIcon} />
                </Pressable>

                <MediaModal
                    image={image}
                    isModalOpen={isModalOpen}
                    setIsModalOpen={setIsModalOpen}
                    setImage={setImage}
                />
            </>
            }
        />
    )

    function confirmUpload(patientID, image) {
        Alert.alert(
            "Confirm Upload",
            "Are you sure you want to upload this image?",
            [
                {
                    text: "Yes",
                    onPress: () => {
                        insertDocumentFetch(patientID, image).then((response) => {
                            if(!response.success){
                                console.log("Failed to insert document: ", response.message);
                                return;
                            }
                            console.log("Successfully inserted document: ", response.data);
                        });
                        setIsModalOpen(false);
                    },
                },
                {
                    text: "No",
                    onPress: () => console.log("Image Upload Cancelled."),
                    style: "cancel",
                },
            ],
            { cancelable: true }
        )
    }
}

const style = StyleSheet.create({
    floatingButton: {
        borderWidth: 1,
        borderRadius: 50,
        height: 100,
        width: 100,
        alignSelf: "flex-end",
        margin: 20,
    },
    floatingButtonIcon: {
        justifyContent: "center",
        alignSelf: "center",
    },
})