import {SearchBar} from "@/components/SearchBar";
import {
    ActivityIndicator,
    FlatList,
    Linking,
    Pressable,
    StyleProp,
    StyleSheet,
    Text,
    TextStyle,
    useColorScheme,
    View
} from "react-native";
import {Colors} from "@/constants/Colors";
import React, {useEffect, useState} from "react";
import {usePatient} from "@/hooks/usePatient";
import {useFunction} from "@/constants/fetchData";
import {Card} from "@/components/Card";
import Constants from "expo-constants";

export default function Education() {
    const themeStyle = useColorScheme() === "dark" ? Colors.dark : Colors.light;
    const { patientID } = usePatient()
    const [symptomData, setSymptomData] = useState([]);
    useFunction(patientID, setSymptomData, "getSymptoms");
    const [medicationData, setMedicationData] = useState([]);
    useFunction(patientID, setMedicationData, "getMedications");

    const [search, setSearch] = useState("");
    const [loading, setLoading] = useState(true);
    const [answer, setAnswer] = useState([]);

    function handleSearch(search, setAnswer, setLoading){
        fetchGoogle(search)
            .then((response) => {
                setLoading(false);
                setAnswer(response["organic_results"]);
            })
            .catch((error) => console.log(error))
    }

    async function fetchGoogle(search){
        setLoading(true);
        const googleApiKey = Constants.expoConfig?.extra?.REACT_APP_GOOGLE_API_KEY;
        const response = await fetch(
            `https://serpapi.com/search.json?q=${encodeURIComponent(search)}&hl=en&gl=us&google_domain=google.com&api_key=${googleApiKey}`,
            { method: "GET" }
        );
        return await response.json();
    }

    useEffect(() => {
        setSearch("");
    }, [patientID]);

    const titleStyle: StyleProp<TextStyle> = {
        fontSize: 18,
        color: themeStyle.color,
        fontWeight: "bold",
        textDecorationLine: "underline",
    };

    const textStyle: StyleProp<TextStyle> = {
        fontSize: 16,
        color: themeStyle.color,
    };

    return(
        <FlatList
            ListHeaderComponent={
            <>
                <SearchBar
                    icon={true}
                    placeholder={"Search Topic To Research."}
                    value={search}
                    onChangeText={setSearch}
                    onBlur={() => handleSearch(search, setAnswer, setLoading)}
                />

                {search ? (
                    <ActivityIndicator
                        size={"small"}
                        animating={loading}
                    />
                ): (
                    <>
                        {symptomData.length > 0 ? <Text style={titleStyle}>Symptoms</Text> : null}
                        {symptomData.map((symptom, _) => (
                            <View key={`symptom-${symptom["symptomID"]}`} >
                                <Card
                                    image={"assessment"}
                                    item1={symptom["symptomTitle"]}
                                    item2={symptom["symptomDescription"]}
                                    quantity={symptom["symptomSeverity"]}
                                    label={"Severity"}
                                />
                            </View>
                        ))}
                        {medicationData.length > 0 ? <Text style={titleStyle}>Medication</Text> : null}
                        {medicationData.map((medication, _) => (
                            <View key={`medication-${medication["medicationID"]}`} >
                                <Card
                                    image={"medication"}
                                    item1={medication["medicationName"]}
                                    item2={medication["description"]}
                                    item3={medication["strength"]}
                                    quantity={medication["quantityAvailable"]}
                                    label={"Severity"}
                                />
                            </View>
                        ))}
                    </>
                )}
            </>
            }
            nestedScrollEnabled={true}
            style={themeStyle}
            data={answer}
            keyExtractor={(item) => item.link}
            renderItem={({ item }) => (
                <View style={style.view}>
                    <Text style={titleStyle}>{item.title}</Text>
                    <Text style={textStyle}>{item["snippet"]}</Text>
                    <Pressable onPress={() => Linking.openURL(item.link)}>
                        <Text style={style.link}>{item.link}</Text>
                    </Pressable>
                </View>
            )}
        />
    )
}

const style = StyleSheet.create({
view: {
    borderWidth: 1,
    borderRadius: 20,
    margin: "1%",
    padding: "2%",
    alignItems: "center",
},
link: {
    color: "blue",
    textDecorationLine: "underline",
}
})