import {
    ActivityIndicator,
    FlatList,
    KeyboardAvoidingView,
    SafeAreaView,
    StyleProp,
    Text,
    TextStyle,
    useColorScheme,
    View,
    ViewStyle,
} from 'react-native';
import React, {useEffect, useState} from "react";
import MapView, {Marker} from 'react-native-maps';
import MapViewDirections from "react-native-maps-directions";
import LocationData from "../components/Location.tsx";
import {Colors} from "@/constants/Colors";
import {SearchBar} from "@/components/SearchBar";
import {usePatient} from "@/hooks/usePatient";
import {Picker} from "@react-native-picker/picker";
import {fetchRequest, useFunction} from "@/constants/fetchData";
import Constants from 'expo-constants';

interface MarkerType {
    coordinate: {
        latitude: number;
        longitude: number;
    };
    key: string;
}

const MapAPIKey = Constants.expoConfig?.extra?.REACT_APP_MAP_API_KEY;

export default function Location() {
    const themeStyle = useColorScheme() === "dark" ? Colors.dark : Colors.light;
    const {patientID} = usePatient();

    const mapSize: StyleProp<ViewStyle> = {
        height: "50%",
        width: "100%",
    };

    const instructionsContainer = {
        paddingTop: 0,
        textAlign: "center",
        backgroundColor: themeStyle.backgroundColor,
    }

    const instructionsText: StyleProp<TextStyle> = {
        fontSize: 14,
        textAlign: "left",
        color: themeStyle.color,
    }

    const travelTypesContainer = {
        borderRadius: 10,
        borderWidth: 1,
        color: themeStyle.color,
        backgroundColor: themeStyle.backgroundColor,
    }

    const travelTypes = {
        color: themeStyle.color,
        backgroundColor: themeStyle.backgroundColor,
    }

    const travelStats: StyleProp<TextStyle> = {
        paddingTop: 5,
        fontSize: 15,
        textAlign: "center",
        fontWeight: "900",
        color: themeStyle.color,
    }

    const location = LocationData();
    const INITIAL_REGION = {
        latitude: location ? location.coords.latitude : 55.860916,
        longitude: location ? location.coords.longitude : -4.251433,
        latitudeDelta: 0.0922,
        longitudeDelta: 0.0421,
    };

    let [search, setSearch] = useState("");
    let [loading, setLoading ] = useState(false);

    let [locations, setLocations ] = useState();

    let [selectedTravelType, setSelectedTravelType] = useState();
    let [distance, setDistance] = useState(0);
    let [time, setTime] = useState(0);
    let [speed, setSpeed] = useState(0);

    useEffect(() => {
        if (!isNaN(distance / time)) {
            setSpeed(distance / time);
        }
        setSpeed(0);
    }, [distance, time]);

    const [markers, setMarkers] = useState<MarkerType[]>([]);
    const setMarker = (event:any) => {
        if (markers.length < 2) {
            const { coordinate } = event.nativeEvent;
            const newMarker: MarkerType = {
                coordinate,
                key: Math.random().toString(),
            };
            setMarkers([...markers, newMarker]);
        }
    }

    const handleMarkerDrag = (index: number, coordinate: { latitude: number; longitude: number }) => {
        const updatedMarkers = [...markers];
        updatedMarkers[index].coordinate = coordinate;
        setMarkers(updatedMarkers);
    };

    async function getLocationsFetch(patientID) {
        const formData = new FormData();
        formData.append("action", "getLocations");
        formData.append("patientID", patientID);
        return await fetchRequest(formData);
    }

    function getLocations(patientID){
        getLocationsFetch(patientID)
            .then((response) => {
                console.log(response);
            })
            .catch((error) => {
                console.log(error);
            })
    }

    useFunction(patientID, setLocations, "getAppointments");
    function verifySearch(search){
        setLoading(true);
        getRoute(search)
            .then(response => {
                const startMarker = {
                    coordinate: { latitude: INITIAL_REGION.latitude, longitude: INITIAL_REGION.longitude },
                    key: Math.random().toString(),
                };
                const endMarker = {
                    coordinate: { latitude: response.lat, longitude: response.lon },
                    key: Math.random().toString(),
                };
                setMarkers([startMarker, endMarker]);
                setLoading(false);
            });
    }

    async function getRoute(location){
        const response = await fetch(`https://maps.googleapis.com/maps/api/geocode/json?key=${MapAPIKey}&address=${encodeURIComponent(location)}`);
        const data = await response.json();
        return {
            lat: data.results[0].geometry["location"].lat,
            lon: data.results[0].geometry["location"].lng,
        }
    }

    useEffect(() => {
        setSearch("");
    }, [patientID]);

    return (
        <View style={[themeStyle, {flex: 1}]}>
            <SearchBar
                icon={true}
                placeholder={"Search Location."}
                value={search}
                onChangeText={setSearch}
                onBlur={() => verifySearch(search)}
            />

            {locations ? (
                <KeyboardAvoidingView>
                    <FlatList
                        data={locations}
                        renderItem={({item}) => (
                            <Text style={[themeStyle, {alignSelf: "center"}]}>- {item.location}</Text>
                        )}
                    />
                </KeyboardAvoidingView>
            ): null}

            {search ? (
                <ActivityIndicator
                    size={"small"}
                    animating={loading}
                />
            ): null}

            <SafeAreaView style={instructionsContainer}>
                    <Text style={instructionsText}>- Long hold on the map to place up to 2 markers.</Text>
                    <Text style={instructionsText}>- Long hold a marker to pick it up and re-distribute</Text>
                    <Text style={instructionsText}>- The route will automatically be calculated</Text>
                    <Text style={instructionsText}>- Click on a marker to delete it</Text>
                <Picker
                    selectedValue={selectedTravelType}
                    onValueChange={setSelectedTravelType}
                    style={travelTypesContainer}
                >
                    <Picker.Item label={"DRIVING"} value="DRIVING" style={travelTypes} enabled={true}/>
                    <Picker.Item label={"BICYCLING"} value="BICYCLING" style={travelTypes}/>
                    <Picker.Item label={"WALKING"} value="WALKING" style={travelTypes}/>
                    <Picker.Item label={"TRANSIT"} value="TRANSIT" style={travelTypes}/>
                </Picker>
            </SafeAreaView>

            <MapView
                onLongPress={setMarker}
                initialRegion={INITIAL_REGION}
                style={mapSize}
                showsUserLocation={true}
                followsUserLocation={true}
                showsCompass={true}
                showsPointsOfInterest={true}
            >
                {markers.map((marker, index) => (
                    <Marker
                        onPress={() => setMarkers(prevMarkers => prevMarkers.filter(m => m.key !== marker.key))}
                        draggable={true}
                        key={marker.key}
                        coordinate={marker.coordinate}
                        onDragEnd={(event) => handleMarkerDrag(index, event.nativeEvent.coordinate)}
                    />
                ))}
                {markers.length === 2 && (
                    <MapViewDirections
                        origin={markers[0].coordinate}
                        destination={markers[1].coordinate}
                        apikey={MapAPIKey}
                        mode={selectedTravelType}
                        strokeWidth={3}
                        strokeColor="green"
                        onError={(errorMessage) => {
                            console.log(errorMessage);
                        }}
                        onReady={result => {
                            setTime(result.duration);
                            setDistance(result.distance);
                        }}
                    />
                )}
            </MapView>
            {markers.length == 2 ? (
                <Text style={travelStats}>Distance: {Math.round(distance)} km | Time: {Math.round(time)} minutes | Speed: {Math.round(speed)} km/m</Text>
            ) : null}
        </View>
    );
}

